"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const bodyparser = require("koa-bodyparser");
const mount = require("koa-mount");
const openapi_backend_1 = require("openapi-backend");
const commonFlags = require("../common/flags");
const koa_1 = require("../common/koa");
const swagger_ui_1 = require("../common/swagger-ui");
const definition_1 = require("../common/definition");
class Mock extends command_1.Command {
    async run() {
        const { args, flags } = this.parse(Mock);
        const { port, logger, 'swagger-ui': swaggerui } = flags;
        const definition = definition_1.resolveDefinition(args.definition);
        if (!definition) {
            this.error('Please load a definition file', { exit: 1 });
        }
        const api = new openapi_backend_1.default({ definition });
        api.register({
            validationFail: (c, ctx) => {
                ctx.status = 400;
                ctx.body = { err: c.validation.errors };
            },
            notFound: (c, ctx) => {
                ctx.status = 404;
                ctx.body = { err: 'not found' };
            },
            notImplemented: (c, ctx) => {
                const { status, mock } = c.api.mockResponseForOperation(c.operation.operationId);
                ctx.status = status;
                ctx.body = mock;
            },
        });
        await api.init();
        const app = koa_1.createServer({ logger });
        app.use(bodyparser());
        // serve openapi.json
        const openApiFile = 'openapi.json';
        const documentPath = `/${openApiFile}`;
        app.use(mount(documentPath, async (ctx, next) => {
            await next();
            const doc = api.document;
            doc.servers = [
                {
                    url: `http://localhost:${port}`,
                },
            ];
            ctx.body = api.document;
            ctx.status = 200;
        }));
        // serve swagger ui
        if (swaggerui) {
            app.use(mount(`/${swaggerui}`, swagger_ui_1.serveSwaggerUI({ url: documentPath })));
        }
        // serve openapi-backend
        app.use((ctx) => api.handleRequest({
            method: ctx.request.method,
            path: ctx.request.path,
            body: ctx.request.body,
            query: ctx.request.query,
            headers: ctx.request.headers,
        }, ctx));
        // start server
        const { port: portRunning } = await koa_1.startServer({ app, port });
        this.log();
        this.log(`Mock server running at http://localhost:${portRunning}`);
        if (swaggerui) {
            this.log(`Swagger UI running at http://localhost:${portRunning}/${swaggerui}`);
        }
        this.log(`OpenAPI definition at http://localhost:${portRunning}${documentPath}`);
        this.log();
    }
}
Mock.description = 'start a local mock API server';
Mock.examples = [
    '$ openapi mock ./openapi.yml',
    '$ openapi mock https://raw.githubusercontent.com/OAI/OpenAPI-Specification/master/examples/v3.0/petstore.yaml',
];
Mock.flags = Object.assign({}, commonFlags.help(), commonFlags.serverOpts(), { 'swagger-ui': command_1.flags.string({ char: 'U', description: 'Swagger UI endpoint', helpValue: 'docs' }) });
Mock.args = [
    {
        name: 'definition',
        description: 'input definition file',
    },
];
exports.default = Mock;
