"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const SwaggerParser = require("swagger-parser");
const YAML = require("js-yaml");
const fs = require("fs");
const path = require("path");
const os_1 = require("os");
exports.CONFIG_FILENAME = '.openapiconfig';
async function parseDefinition({ definition, dereference, validate, bundle, servers, }) {
    let method = SwaggerParser.parse;
    if (bundle) {
        method = SwaggerParser.bundle;
    }
    if (dereference) {
        method = SwaggerParser.dereference;
    }
    if (validate) {
        method = SwaggerParser.validate;
    }
    const document = await method.bind(SwaggerParser)(definition);
    // add servers
    if (servers) {
        const serverObjects = servers.map((url) => ({ url }));
        document.servers = document.servers ? [...document.servers, ...serverObjects] : serverObjects;
    }
    return document;
}
exports.parseDefinition = parseDefinition;
var OutputFormat;
(function (OutputFormat) {
    OutputFormat["JSON"] = "json";
    OutputFormat["YAML"] = "yaml";
})(OutputFormat = exports.OutputFormat || (exports.OutputFormat = {}));
function stringifyDocument({ document, format }) {
    if (format === OutputFormat.JSON) {
        // JSON output
        return JSON.stringify(document, null, 2);
    }
    else {
        // YAML output
        return YAML.safeDump(document, { noRefs: true, lineWidth: 240, noArrayIndent: true });
    }
}
exports.stringifyDocument = stringifyDocument;
// walk backwards from cwd until homedir and check if CONFIG_FILENAME exists
function resolveConfigFile() {
    let dir = path.resolve(process.cwd());
    while (dir.length >= os_1.homedir().length) {
        const check = path.join(dir, exports.CONFIG_FILENAME);
        if (fs.existsSync(check)) {
            return path.join(dir, exports.CONFIG_FILENAME);
        }
        else {
            // walk backwards
            dir = path.resolve(path.join(dir, '..'));
        }
    }
}
exports.resolveConfigFile = resolveConfigFile;
function resolveDefinition(definitionArg) {
    // check definitionArg
    if (definitionArg && definitionArg !== 'CURRENT') {
        return definitionArg;
    }
    if (process.env.OPENAPI_DEFINITION && definitionArg !== 'CURRENT') {
        return process.env.OPENAPI_DEFINITION;
    }
    const configFile = resolveConfigFile();
    if (configFile) {
        const config = YAML.safeLoad(fs.readFileSync(configFile));
        return config.definition;
    }
}
exports.resolveDefinition = resolveDefinition;
function printInfo(document, ctx) {
    const { title, version, description } = document.info;
    ctx.log(`title: ${title}`);
    ctx.log(`version: ${version}`);
    if (description) {
        ctx.log(`description: ${description}`);
    }
}
exports.printInfo = printInfo;
function printOperations(document, ctx) {
    const operations = {};
    if (document.tags) {
        for (const tag of document.tags) {
            const { name, description } = tag;
            operations[name] = {
                description,
                routes: [],
            };
        }
    }
    for (const path in document.paths) {
        if (document.paths[path]) {
            for (const method in document.paths[path]) {
                if (document.paths[path][method]) {
                    const { operationId, summary, description, tags } = document.paths[path][method];
                    let route = `${method.toUpperCase()} ${path}`;
                    if (summary) {
                        route = `${route} - ${summary || description}`;
                    }
                    if (operationId) {
                        route = `${route} (${operationId})`;
                    }
                    for (const tag of tags || ['default']) {
                        if (!operations[tag]) {
                            operations[tag] = { routes: [] };
                        }
                        operations[tag].routes.push(route);
                    }
                }
            }
        }
    }
    ctx.log('Operations:');
    for (const tag in operations) {
        if (operations[tag]) {
            const routes = operations[tag].routes;
            for (const route of routes) {
                ctx.log(`- ${route}`);
            }
        }
    }
}
exports.printOperations = printOperations;
function printSchemas(document, ctx) {
    const schemas = (document.components && document.components.schemas) || {};
    const count = Object.entries(schemas).length;
    if (count > 0) {
        ctx.log(`Schemas (${count}):`);
        for (const schema in schemas) {
            if (schemas[schema]) {
                ctx.log(`- ${schema}`);
            }
        }
    }
}
exports.printSchemas = printSchemas;
